%   ==========================================================
%   Computation of periodic solutions of a one dof system with
%   essential cubic nonlinearity (no linear part)
%   ==========================================================
%
%   Initial dynamical system:
%   -------------------------
%
%   (1)   u'' + lambda u' + Gamma u^3 =  0
%
%   u(t) is a real unknown time function. The other parameters are
%   real.
%
%   First order dynamical system
%   ----------------------------
%
%   We introduce:
%
%         v=u' 
%
%   to transform (1) into the first order dynamical system:
%
%   (2)   u' =  v
%         v' = - lambda v - Gamma u^3
%
%   The Jacobian writes:
%
%         J(t) = [ 0             1;
%                 -3*Gamma*u^2  -mu];
%
%   Quadratic dynamical system
%   --------------------------
%
%   We introduce 
%
%         v=u' 
%         w=u^2
%
%   to transform (1) into the first order system with quadratic nonlinearities:
%
%   (3)   u' =  v
%         v' =     - lambda v - Gamma uw
%         0  =  w             - u^2
%   
%   with Neq=3 equations
%
%   Small operators formalism
%   -------------------------
% 
%   The following formalism is introduced:
%
%         m(u') = c0 + l0(u) + lambda l1(u) + q(u,u)
%
%   where: 
%
%         u(t)=[u v w]
%         c0 is a constant operator in vector u
%         l1(u), l2(u) are linear operators in vector u
%         q(u,u) is a quadratic operator in vector u
%
%   Harmonic Balance Method (HBM)
%   -----------------------------
% 
%   The Harmonic Balance method (HBM) leads to seak vector u(t) in a
%   Fourier series truncated to H harmonics:
%
%        u(t) = u0 + 
%               u1c cos(omega t) + u1s sin(omega t) + 
%               u2c cos(2 Omega t) + u2s sin(2 Omega t) + 
%               ... +
%               uHc cos(H omega t) + uHs sin(H omega t)
%
%   which leads to a nonlinear algebraic system of Neq*(2*H+1)
%   equations of the form:
%
%        C0  + L0(U) + lambda L1(U)  + Q(U,U) - omega M(U) = 0 
%
%   where 
%  
%        U=[u0 u1c u1s  u2c u2s  ...  uHs]
%  
%   Final algebraic system for Manlab  
%   ---------------------------------
% 
%        R(bU) = bL0 + bL(bU) + bQ(bU,bU) =0 
%
%   with
%
%        bU = [U lambda omega]
%
%        bL0       = C0
%        bL(bU)    = L0(U) + lambda C1
%        bQ(bU,bU) = Q(U,U) + lambda L1(U) - omega M(U)
%
%   Computing bL0, bL, and bQ as functions of c0, l0, l1, m and q is
%   automatic.
%
%   O. Thomas, Oct 2010
%
% ====================================================================


% Path for the Manlab sources
addpath('/home/sami/MANLAB/SRC')

close all
clear all
clear classes
% Manlab initialisation
manlabinit;

% Fortran acceleration
% --------------------

% use the following line to use Fortran acceleration
[status,result]=unix('cd @DUFFESS && make && cd ..','-echo');
% or
% use the following to remove Fortran acceleration
%[status,result]=unix('cd @DUFFESS && make clean && cd ..','-echo');
 

% Parameters
% ----------
 
% Generic for HBM 
H= 30;        % Number of harmonics
Neq=3;        % Number of equations of the quadratic system   

% Particular for this example
Gamma=1;   % NL coefficient
	   
% Object creation
% ---------------

ML_problem  = DUFFESS(H,Neq,Gamma); 
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('% ! WARNING !  : check the hardly modified Fortran parameters %')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
% !!! WARNING !!! 
% If using Fortran, the equation parameters are not given to the
% .f90 files and must be hardly modified in the preamble of
% the file 'petits_operateurs.f90' 

% Stability computation parameters
% --------------------------------

ML_stability = 1; % 1/0: stability computation activation/desactivation
ML_algostab = 2;  % 1 -> time domain algorithm
                  % 2 -> frequency domain algorithm

% Entry vector for stability computations:
% which variables are needed for the Jacobian
% ML_varstab=[Ustab lambda omega H];
% Ustab -> indices in the small vector u
% lambda -> Neq*(2*H+1)+1 (indices in big vector bU)
% omega -> Neq*(2*H+1)+2 (indices in big vector bU)
ML_varstab = [1 Neq*(2*H+1)+1 Neq*(2*H+1)+2 H]; 

% Tolerance on the stability check
% |rho| > 1+tolstab -> unstable
ML_tolstab=1e-3;
% Order of the power series order for ANM branches
ML_problem = set_ordre(ML_problem,15);         
% number of point in a ANM branch for display and stability computation
ML_problem = set_nbptstroncon(ML_problem,20);
% number of time sample in a period for time domain stability computation 
% (only used if ML_algostab=1)
ML_problem = set_nech(ML_problem,500);        

% Starting vector for the initial Newton-Raphson algorithm
% --------------------------------------------------------

% H=1 harmonic balance approximate solution
a=0.01;

% u=a*cos(omega t)
% v=-omega*a*sin(omega t)
% w=u^2=0.5*a^2*(1+cos(2 omega t))
 
omega=sqrt(3/4*Gamma)*a; % 1st order HBM
u=a;v=-omega*a;w=0.5*a^2;

ML_Ustart = zeros(Neq*(2*H+1)+2,1);
ML_Ustart(1:Neq)=[0  0 w]; % H0
ML_Ustart(Neq+1:2*Neq)=[u 0 0]; % H1 in cos
ML_Ustart(2*Neq+1:3*Neq)=[0 v 0]; % H1 in sin
ML_Ustart(3*Neq+1:4*Neq)=[0 0 w]; % H2 in cos
ML_Ustart(Neq*(2*H+1)+1)=1e-12; % lambda
ML_Ustart(Neq*(2*H+1)+2)=omega; % omega
 
% Indices of variables to be displayed on the main bifurcation
% diagram of figure(2) 
% -------------------------------------------------------------- 

% ML_dispvars = [x1 y1 ; x2 y2; ...]
% omega -> (2*H+1)*Neq+1

ML_dispvars = [Neq+1   (2*H+1)*Neq+2 ; % u H1c , omega
%	       Neq+2   (2*H+1)*Neq+2 ; % v H1c , omega
%		2*Neq+1 (2*H+1)*Neq+2 ; % u H1s , omega
%		2*Neq+2 (2*H+1)*Neq+2 ; % v H1s , omega
	       (2*H+1)*Neq+1 (2*H+1)*Neq+2 ]; % lambda , omega

% Activation of point display and global display
% ----------------------------------------------

ML_pointdisplay = 0;
ML_globaldisplay = 0;
 
% Manlab start
% ------------

manlabstart; 


