% ===================================================
% Resonance curve of a prestressed hinged hinged beam
%
% Particular example of:
%   R. Lewandowski, 1997, International Journal 
%   of Solids and Structures, 34(15), pp. 1949-1967
% ===================================================
% 
% Model equations
% ---------------
%
% The transverse displacement writes:
%
% w(x,t) = Sum_{k=1}^K Phi_k(x) uk(t)
%
% with 
%   * Phi_k(x) = \sqrt{2} sin(k pi x/L)
%     the k-th mode shape of the hinged-hinged beam 
%   * uk(t) the k-th modal coordinate
%
% Dimensionless Discretized model
% -------------------------------
% 
% For all k \in [1,...,K]
%
% uk'' + 2muk uk' + omegak2 uk + epsilon k^2 pi^2 N uk = F cos Omega t
% N = N0 + \sum_k k^2 pi^2 uk^2
%
%   * \epsilon = 6 for a prismatic cross section
%   * N0 is the prestress
%   * N is the axial force 
%
% For Manlab:
% -----------
%
% For all k \in [1,...,K]
%
% uk' =                 vk
% vk' = F cos Omega t - 2muk vk - omegak2 uk  - epsilon k^2 pi^2 N uk 
% 0   = N0            - N                     + \sum k^2 pi^2 uk^2
%
% 
%  With  Z(t) = [ u1 , ... , uK , v1 , ... vK , N]  
%  (Neq=2K+1) the system becomes
%  
%  m(Z') = c(Omega,t)  + l(Z)  + q(Z,Z) 
%
%  Harmonic balance
%  ----------------
%
%  Z(t) =  Z0 
%        + Z1c cos(omega t) + Z1s sin(omega t)  
%        + Z2c cos(2 Omega t) + Z2s sin(2 Omega t) 
%        + ... 
%        + ZHc cos(H omega t) + ZHs sin(H omega t).
%
%  the balance of the harm onics leads to a large system of size  Neq*(2H+1)
%
%      C0    +   L0(U)   + Q(U,U) - Omega M(U)= 0 
%                  
%
%   with    U=[Z0  Z1c Z1s  Z2c Z2s  ...  ZHs ]
% 
%   no phase condition
%
%  Final algebraic system for Manlab  bL0 + bL(bU) + bQ(bU,bU) =0 
%  --------------------------------------------------------------
%
%   bU = [ U , Omega ]
%
%   bL0       = C0
%   bL(bu)    = L0(U) + lambda C1
%   bQ(bU,bU) = Q(U,U) + lambda L1(U) - Omega M(U)
%
%
%
%   O. Thomas, A. Lazarus, 2010
%
%--------------------------------------------------------------------------


% Path for the Manlab sources
addpath('/home/thomas/Matlab/MANLAB/MANLAB-2.0-beta/SRC')
addpath('/home/thomas/Matlab/MANLAB/MANLAB-2.0-beta/SRC_VISU')

clear classes
clear all
% Manlab initialisation
manlabinit;

% Fortran acceleration
% ====================

% use the following line to use Fortran acceleration
[status,result]=unix('cd @VIBBEAMARTART && make && cd ..','-echo');
% or
% use the following to remove Fortran acceleration
%[status,result]=unix('cd @VIBBEAMARTART && make clean && cd ..','-echo');

% Parameters
% ==========
 
H=10;        % Number of harmonics
disp(['H=',num2str(H)])
K=3;          % Number of retained modes in the Galerkin expansion
disp(['K=',num2str(K)])
%  Number of equations for all harminics block
Neq=2*K+1;
disp(['Neq=',num2str(Neq)])
% Final vector U size:
Nu=Neq*(2*H+1)+1;
disp(['Nu=',num2str(Nu)])



% Natural frequencies
kk=(1:K)';
OMEG=(pi*kk).^2;
disp(['OMEG=',num2str(OMEG',10)])

% Beam parameters
% ---------------

E=100e9; h=1e-3; b=0.1; L=1;

S=h*b; I=b*h^3/12;E=7e10;rho=2000;
disp(['Beam: ',num2str(1000*h),'x',num2str(1000*b),'x',num2str(1000*L),' mm'])
% Prismatic cross section: \epsilon=6
epsil=6;
disp(['epsil=',num2str(epsil)])

% Precontrainte
% -------------

%N0=(-pi^4+.001)/epsil/pi^2;       % precontrainte [adim.]
N0 = 0;
disp(['N0=',num2str(N0)])
N0dim=N0*E*S*h^2/2/L^2;
disp(['Prestress: ',num2str(N0dim),' N'])

% Vecteur des facteur d'amortissements
% ------------------------------------

% Proportional damping: \xi_k=0.5*(alpha/\omega_k+\beta\omega_k)
xi = 0.005;    % damping factor
alpha = xi*2*OMEG(1); % Lewandowski_IJSS1997
beta = 0; % Lewandowski_IJSS1997
XI = 0.5*(alpha./OMEG+beta.*OMEG);
disp(['XI=',num2str(XI',10)])

% Vecteur de forcage
% ------------------

% To opposing phase forces: P1 at L/4 and P2  3L/4 (Lewandowski_IJSS1997)
r=h/sqrt(12); % Rayon de gyration r=sqrt(I/A)
xF1=.25;
P1dim=13.63*E*I*r/L^3;
P1=P1dim/(E*I*h/L^3);
xF2=.75;
P1dim=9.62*E*I*r/L^3;
P2=P1dim/(E*I*h/L^3);
Qext1=-sqrt(2)*P1*sin(kk*pi*xF1); Qext2=sqrt(2)*P2*sin(kk*pi*xF2);Qext=Qext1+Qext2;

disp(['Qext=',num2str(Qext',10)])

% Object creation
% ===============

ML_problem  = VIBBEAMARTART(H,Neq,N0,XI,epsil,Qext);
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('% ! WARNING !  : check the hardly modified Fortran parameters %')
disp('%                check the hardly modified L0.F90 function    %')
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
% !!! WARNING !!! 
% If using Fortran, the equation parameters are not given to the
% .f90 files and must be hardly modified in the preamble of
% the file 'petits_operateurs.f90' 

% Stability computation parameters
% --------------------------------

ML_stability = 1; % 1/0: stability computation activation/desactivation
ML_algostab = 2;  % 1 -> time domain algorithm
                  % 2 -> frequency domain algorithm

% Entry vector for stability computations:
% which variables are needed for the Jacobian
% ML_varstab=[Ustab lambda omega H];
% Ustab -> indices in the small vector u
% Here (non autonomous system), continuation parameter lambda = omega
% omega -> Neq*(2*H+1)+1 (indices in big vector bU)
ML_varstab = [1:K  Neq  Neq*(2*H+1)+1 Neq*(2*H+1)+1 H];

% Tolerance on the stability check
% |rho| > 1+tolstab -> unstable
ML_tolstab=0;
% Order of the power series order for ANM branches
ML_problem = set_ordre(ML_problem,15);         
% number of point in a ANM branch for display and stability computation
ML_problem = set_nbptstroncon(ML_problem,20);
% number of time sample in a period for time domain stability computation 
% (only used if ML_algostab=1)
ML_problem = set_nech(ML_problem,1000);        

% Starting vector for the initial Newton-Raphson algorithm
% --------------------------------------------------------
ML_Ustart   = [zeros(Nu-1,1) ; 2.34];  
%ML_Ustart(Neq+1)=-0.1568;
%ML_Ustart(end)=2.3404;


% Indices of variables to be displayed on the main bifurcation
% diagram of figure(2) 
% -------------------------------------------------------------- 

% Numros des coordonnes modales a afficher
kkdisp=[Neq+(1:K) 2*Neq+(1:K)];
%kkdisp=[Neq+1 2*Neq+1];

% ML_dispvars = [x1 y1 ; x2 y2; ...]
% omega -> Nu
ML_dispvars = [Nu*ones(size(kkdisp')) kkdisp']; % Cosine and Sine
                                                % of H1 for each
                                                % retained modal coordinate

% Activation of point display and global display
% ----------------------------------------------

ML_pointdisplay = 0;
ML_globaldisplay = 0;
 
% Manlab start
% ------------

global CPUTIME
CPUTIME = [];
tic
manlabstart;              