%   ==============================================================
%   Computation of periodic solutions of a 2 ddl system with cubic
%   nonlinearities, Eq. (2) of:
%   Kerschen et al., 2009, Mechanical Systems and Signal
%   Processing, 23(1), pp. 170-194
%
%   Computation of the conserved ENERGY as a degree of freedom
%   ==============================================================
%
%   Initial dynamical system:
%   -------------------------
%
%   (1)   u1'' + lambda u1 + 2 u1 - u2 + 0.5 u1^3 = 0
%         u2'' + lambda u2 + 2 u2 - u1 = 0
%
%   u1(t) et u2(t) are two real unknown time functions. The other parameters are
%   real.
%
%   First order dynamical system
%   ----------------------------
%
%   We introduce:
%         v1 = u1'
%         v2 = u2'
%
%   to transform (1) into the first order dynamical system:
%
%    (2)  u1'  =     v1
%         u2'  =     v2
%         v1'  =    -2*u1 + u2 -lambda v1 - 0.5* u1^3   
%         v2'  =    -2*u2 + u1 -lambda v2
%
%   The Jacobian writes:
%
%         J(t) = [ 0             0   1       0;
%                  0             0   0       1;
%                 -2 -3/2*u1^2   1  -lambda  0;
%                  1            -2   0      -lambda;]
%
%   Quadratic dynamical system
%   --------------------------
%
%   We introduce 
%
%         v1 = u1'
%         v2 = u2'
%         w1 = u^2
%          h = 0.5*(v1^2+v2^2+u1^2+u2^2+(u1-u2)^2-1/4u1^4)
%
%   with h the conserved energy of the system.
%   
%   We obtain the following first order system with quadratic nonlinearities:
%
%   (3)   u1'  =     v1
%         u2'  =     v2
%         v1'  =    -2*u1 + u2 -lambda v1  - 0.5* u1 w1   
%         v2'  =    -2*u2 + u1 -lambda v2
%         0    =     w1                    - u1^2
%         0    =     h                     - (0.5*(v1^2+v2^2)+u1^2+u2^2-u1*u2-1/8*w1^2)
%   
%   with Neq=6 equations
%
%   Small operators formalism
%   -------------------------
% 
%   The following formalism is introduced:
%
%         m(u') = c0 + l0(u) + lambda l1(u) + q(u,u)
%
%   where: 
%
%         u(t)=[u1 v1 u2 v2 w1]
%         c0 is a constant operator in vector u
%         l1(u), l2(u) are linear operators in vector u
%         q(u,u) is a quadratic operator in vector u
%
%   Harmonic Balance Method (HBM)
%   -----------------------------
% 
%   The Harmonic Balance method (HBM) leads to seak vector u(t) in a
%   Fourier series truncated to H harmonics:
%
%        u(t) = u0 + 
%               u1c cos(omega t) + u1s sin(omega t) + 
%               u2c cos(2 Omega t) + u2s sin(2 Omega t) + 
%               ... +
%               uHc cos(H omega t) + uHs sin(H omega t)
%
%   which leads to a nonlinear algebraic system of Neq*(2*H+1)
%   equations of the form:
%
%        C0  + L0(U) + lambda L1(U)  + Q(U,U) - omega M(U) = 0 
%
%   where 
%  
%        U=[u0 u1c u1s  u2c u2s  ...  uHs]
%  
%   Final algebraic system for Manlab  
%   ---------------------------------
% 
%        R(bU) = bL0 + bL(bU) + bQ(bU,bU) =0 
%
%   with
%
%        bU = [U lambda omega]
%
%        bL0       = C0
%        bL(bU)    = L0(U) + lambda C1
%        bQ(bU,bU) = Q(U,U) + lambda L1(U) - omega M(U)
%
%   Computing bL0, bL, and bQ as functions of c0, l0, l1, m and q is
%   automatic.
%
% ====================================================================

 % Path for the Manlab sources

addpath('/home/thomas/Matlab/MANLAB/MANLAB-2.0-beta/SRC')
addpath('/home/thomas/Matlab/MANLAB/MANLAB-2.0-beta/SRC_VISU')

clear classes
clear all
close all
% Manlab initialisation
manlabinit;

% Fortran acceleration
% --------------------

% use the following line to use Fortran acceleration
[status,result]=unix('cd @NNMVAKNRJ && make && cd ..','-echo');
% or
% use the following to remove Fortran acceleration
%[status,result]=unix('cd @NNMVAKNRJ && make clean && cd ..','-echo');
 

% Parameters
% ----------

% Generic for HBM 
H=6;         % Number of harmonics
Neq=6;        % Number of equations of the quadratic system

% Object creation
% ---------------

ML_problem  = NNMVAKNRJ(H,Neq); % creation de l' objet 

% Stability computation parameters
% --------------------------------
 
ML_stability = 1; % 1/0: stability computation activation/desactivation
ML_algostab = 2;  % 1 -> time domain algorithm
                  % 2 -> frequency domain algorithm

% Entry vector for stability computations:
% which variables are needed for the Jacobian
% ML_varstab=[Ustab lambda omega H];
% Ustab  -> indices in small vector u
% lambda -> Neq*(2*H+1)+1 (indice in big vector bU)
% omega  -> Neq*(2*H+1)+2 (indice in big vector bU)
ML_varstab = [1 Neq*(2*H+1)+1 Neq*(2*H+1)+2 H];

% Tolerance on the stability check
% |rho| > 1+tolstab -> unstable
ML_tolstab=1e-3;

% Order of the power series order for ANM branches
ML_problem = set_ordre(ML_problem,15);         
% number of point in a ANM branch for display and stability computation
ML_problem = set_nbptstroncon(ML_problem,20);
% number of time sample in a period for time domain stability computation 
% (only used if ML_algostab=1)
ML_problem = set_nech(ML_problem,1000);        
 
% Starting vector for the initial Newton-Raphson algorithm
% --------------------------------------------------------

% Natural frequencies computation
M=[1 0;0 1];
K=[2 -1;-1 2]; 
[V,Omeg]=eig(K,M);
Omeg=sqrt(diag(Omeg));
[Omeg,I]=sort(Omeg);
V=V(:,I);

% Mode 1: omega1=1, in-phase mode
% Mode 2: omega2=sqrt(3), out-of-phase mode
imode=1;

% H=1 harmonic balance approximate solution
ampl=0.01;
% u1=a1*cos(omega t)
% v1=-omega*a1*sin(omega t)
% w1=u1^2=0.5*a1^2*(1+cos(2 omega t))
% Mode 1: h=a^2
% Mode 2: h=3*a^2

omega=Omeg(imode);
V=V(:,imode);
V=V/max(V)*ampl;
a1=V(1);
a2=V(2);
u1=a1;v1=-omega*a1;w1=0.5*a1^2;
u2=a2;v2=-omega*a2;
if imode==1
  h=ampl^2;
else
  h=3*ampl^2;
end 

ML_Ustart = zeros(Neq*(2*H+1)+2,1);
ML_Ustart(1:Neq)=[ 0   0   0  0 w1 h]; % H0
ML_Ustart(Neq+1:2*Neq)=[u1 u2  0   0  0 0]; % H1 en cos
ML_Ustart(2*Neq+1:3*Neq)=[0  0   v1  v2 0 0]; % H1 en sin
ML_Ustart(3*Neq+1:4*Neq)=[0  0   0   0  w1 0]; % H2 en cos
ML_Ustart(Neq*(2*H+1)+1)=1e-12; % lambda
ML_Ustart(Neq*(2*H+1)+2)=omega; % omega
				 
 
 % Indices of variables to be displayed on the main bifurcation
% diagram of figure(2) 
% -------------------------------------------------------------- 
% ML_dispvars = [Neq+1   (2*H+1)*Neq+2 ; % u1 H1c , omega
%		Neq+2   (2*H+1)*Neq+2 ; % u2 H1c , omega
%		3*Neq+1 (2*H+1)*Neq+2 ; % u1 H2c , omega
%		3*Neq+2 (2*H+1)*Neq+2 ; % u2 H2c , omega
%		5*Neq+1 (2*H+1)*Neq+2 ; % u1 H3c , omega
%		5*Neq+2 (2*H+1)*Neq+2 ; % u2 H3c , omega
%		(2*H+1)*Neq+1 (2*H+1)*Neq+2 ]; % lambda , omega

  ML_dispvars = [6 (2*H+1)*Neq+2]; % h H0 , omega

% Activation of point display and global display
% ----------------------------------------------

ML_pointdisplay = 1;
ML_globaldisplay = 1;
 
% Manlab start
% ------------

manlabstart; 



