%   ===============================================
%   Periodic solutions of a parametrically excited
%   Duffing equation
%   ===============================================
%
%   Initial dynamical system:
%   -------------------------
% 
%   (1)   u'' + mu u' + ome0^2 u + Gamma u^3 + delta (u cos(2 Omega t)) = F cos Omega t  
%
%   u(t) is a real unknown time function. The other parameters are
%   real.
%
%   First order dynamical system
%   ----------------------------
%
%   We introduce:
%
%         v=u' 
%
%   to transform (1) into the first order dynamical system:
%
%   (2)   u' = v
%         v' = - ome0^2 u - mu v - Gamma u^3 + F cos(omega t) + delta u cos(2 Omega t)
%
%   The Jacobian writes:
%
%         J(t) = [ 0                                            1  ;
%                 -ome0^2 -3*Gamma*u^2 +delta*cos(2 Omega t)   -mu];
%
%   Quadratic dynamical system
%   --------------------------
%
%   We introduce 
%
%         v = u' 
%         w = u^2
%         x = cos(2Omega t)
% 
%   to transform (1) into the first order system with quadratic nonlinearities:
%
%     (3) u'  =                     v
%         v'  = F cos(Omega t)     -mu v -omeg0^2 u    -Gamma uw -delta ux    
%         0   =                    -w                   u^2
%         0   = cos(2Omega t)      -x  
%   
%   with Neq=4 equations
%
%   The Jacobian writes:
%
%         J(t) = [ 0                              1  ;
%                 -ome0^2 -3*Gamma*u^2 +delta*x  -mu];
%
%   Small operators formalism
%   -------------------------
% 
%   The following formalism is introduced:
%
%         m(u') = c + l(u) + q(u,u)
%
%   where: 
%
%         u(t)=[u v w x]
%         c is a constant operator in vector u
%         l(u) is a linear operator in vector u
%         q(u,u) is a quadratic operator in vector u
%
%   Harmonic Balance Method (HBM)
%   -----------------------------
% 
%   The Harmonic Balance method (HBM) leads to seak vector u(t) in a
%   Fourier series truncated to H harmonics:
%
%        u(t) = u0 + 
%               u1c cos(omega t) + u1s sin(omega t) + 
%               u2c cos(2 Omega t) + u2s sin(2 Omega t) + 
%               ... +
%               uHc cos(H omega t) + uHs sin(H omega t)
%
%   which leads to a nonlinear algebraic system of Neq*(2*H+1)
%   equations of the form:
%
%        C  + L(U)  - omega M(U) + Q(U,U)  = 0 
%
%   where 
%
%        U=[u0 u1c u1s  u2c u2s  ...  uHs]
%  
%   Final algebraic system for Manlab  
%   ---------------------------------
% 
%        R(bU) = bL0 + bL(bU) + bQ(bU,bU) =0 
%
%   with
%
%        bU = [U omega]
%
%        bL0       = C
%        bL(bU)    = L(U)
%        bQ(bU,bU) = Q(U,U) - omega M(U)
%
%   Computing bL0, bL, and bQ as functions of c, l, m and q is
%   automatic.
%
%   By O. Thomas - Oct. 2010
%
% ====================================================================

% Path for the Manlab sources
addpath('/home/sami/MANLAB/SRC')

clear classes
clear all
% Manlab initialisation
manlabinit;

% Parameters
% ----------
 
% Generic for HBM 
H= 5;        % Number of harmonics
Neq=4;       % Number of equations of the quadratic system   

% Particular for this example

ome0=1;     % Angular frequency
xi=0.05;     % Damping factor
Gamma=1/6;   % NL coefficient
delta=.5;    % Parametric excitation
F=0;         % Direct excitation

ome0sq=ome0^2;  % Square of angular frequency
mu=2*xi*ome0;   % Twice the damping coefficient

% Object creation
% ---------------

ML_problem  = DUFFOPARAM(H,Neq,ome0sq,mu,Gamma,delta,F); 

% Stability computation parameters
% --------------------------------

ML_stability = 1; % 1/0: stability computation activation/desactivation
ML_algostab = 2;  % 1 -> time domain algorithm
                  % 2 -> frequency domain algorithm
		  
% Entry vector for stability computations:
% which variables are needed for the Jacobian
% ML_varstab=[Ustab lambda omega H];
% Ustab -> indices in the small vector u
% Here (non autonomous system), continuation parameter lambda = omega
% omega -> Neq*(2*H+1)+1 (indices in big vector bU)		  
ML_varstab = [1 4 Neq*(2*H+1)+1 Neq*(2*H+1)+1 H];
		  
% Tolerance on the stability check
% |rho| > 1+tolstab -> unstable
ML_tolstab=0;
% Order of the power series order for ANM branches
ML_problem = set_ordre(ML_problem,15);         
% number of point in a ANM branch for display and stability computation
ML_problem = set_nbptstroncon(ML_problem,20);
% number of time sample in a period for time domain stability computation 
% (only used if ML_algostab=1)
ML_problem = set_nech(ML_problem,1000);        
 
% Starting vector for the initial Newton-Raphson algorithm
% --------------------------------------------------------

ML_Ustart  = zeros(Neq*(2*H+1)+1,1);

if F~=0
  % no problems if the external excitation is non zero
  ML_Ustart(end)=0.1;
else
  % Start on the non trival branch
  % Multiple-scale computation
  % See 
  % Nayfeh, A. H. & Mook, D. T. "Nonlinear Oscillations", Wiley, 1979

  C1=(delta/2/ome0^2).^2-(2*xi)^2;
  epssig=5e-3;
  %sigma=-sqrt(C1)+sign(Gamma)*epssig; a=sqrt(4/3/Gamma*(sigma + sqrt(C1)));
  sigma=sqrt(C1)+sign(Gamma)*epssig; a=sqrt(4/3/Gamma*(sigma - sqrt(C1)));
  Omeg=ome0*(2+sigma)/2;
  ML_Ustart(Neq+1)=a; % 1st harmonics in cosine of u
  ML_Ustart(end)=Omeg;
end

% Indices of variables to be displayed on the main bifurcation
% diagram of figure(2) 
% -------------------------------------------------------------- 

% ML_dispvars = [x1 y1 ; x2 y2; ...]
% omega -> (2*H+1)*Neq+1

ML_dispvars = [(2*H+1)*Neq+1  Neq+1   ;  % u H1c
	       (2*H+1)*Neq+1  2*Neq+1 ;  % u H1s
	       (2*H+1)*Neq+1  5*Neq+1 ;  % u H3c
	       (2*H+1)*Neq+1  6*Neq+1 ]; % u H3s

% Activation of point display and global display
% ----------------------------------------------

ML_pointdisplay = 1;
ML_globaldisplay = 0;
 
% Manlab start
% ------------

manlabstart; 



