% detectbifurcHBM   Bifurcation detection.
%    [Bbif,stab]=detectbifurcHBM(Nds,U,Ustab) returns:
%    + in matrix Bbif:
%      - the indices of the bifurcation points in the first row 
%      - the type of bifurcation in the second row :
%         * 1 for a simple bifurcation (saddle-node, pitchfork...)
%         * 2 for a period doubling bifurcation
%         * 3 for Neimark-Sacker bifurcation
%    + in vector stab (with length(stab)=size(U,2)) a 1 if the
%      branch is unstable and a 0 if the branch is stable. It is the
%      same as Ustab(1,:)
%
%    U is the vector of unknowns of the final algebraic system solved
%    by Manlab and Ustab the stability data. 
%
%    Nds is the initial dynamical system size (this is also the number
%    of Floquet multipliers)
%
%    By O. Thomas / Sept. 2010

function [Bbif,stab]=detectbifurcHBM(Nds,U,Ustab)

%  0 1 1 1 1 1  1 0 0 0 0 0 0
%0 1 1 1 1 1 1  0 0 0 0 0 0
%  1 0 0 0 0 0 -1 0 0 0 0 0

if size(Ustab,1)==1
  stab=ones(size(U(1,:)));
  Bbif=NaN;
  return
end

global ML_tolstab

tolstab=ML_tolstab;
disp(['Stability computation tolerance: tolstab = ',num2str(tolstab)])

Npoint=size(Ustab,2);
omega = U(end,:);

ALP=Ustab(2:(Nds+1),:);
ALP=sort(ALP);
T=2*pi./omega;
TT=ones(Nds,1)*T;
RHO=exp(ALP.*TT);
RHO=sort(RHO,1);


EIGinst=zeros(size(ALP));
I=find(abs(RHO)>(1+tolstab));
EIGinst(I)=ones(size(EIGinst(I)));
%EIGinst
Neiginst=sum(EIGinst,1);
stab=ones(1,size(ALP,2));
I=find(Neiginst>0);
stab(I)=zeros(size(stab(I)));

Neigbif=Neiginst(2:end)-Neiginst(1:end-1);
Ibif=find(Neigbif~=0);
Tbif=zeros(1,length(Ibif));

for ii=1:length(Ibif); 
  ibif=Ibif(ii);
  neigbif=Neigbif(ibif);
%  test=EIGinst(:,ibif)-EIGinst(:,ibif+1);
  if neigbif>1
    disp(['lambda = ',num2str(omega(ibif)),' - Neimark-Sacker bifurcation'])
    Tbif(ii)=3;
  else
    Ieiginst_bef=find(EIGinst(:,ibif)==1);
    Ieiginst_aft=find(EIGinst(:,ibif+1)==1);
    remod_bef=real(RHO(Ieiginst_bef,ibif));
    remod_aft=real(RHO(Ieiginst_aft,ibif+1));
    Npos_bef=length(find(remod_bef>0));
    Npos_aft=length(find(remod_aft>0));
    if Npos_bef~=Npos_aft
      disp(['lambda = ',num2str(omega(ibif)),' - Simple bifurcation'])
      Tbif(ii)=1;
    else
      disp(['lambda = ',num2str(omega(ibif)),' - Period doubling bifurcation'])
      Tbif(ii)=2;
    end
  end
end

Bbif=[Ibif;Tbif];