% plotNNMHBM   Nonlinear Normal Mode / Invariant Manifold plot
%    Hand = plotNNMHBM(Neq,H,U,Idisp,Ia,fig) plots the manifold to
%    which the periodic solutions computed by the Manlab/HBM method
%    belongs.
%
%    U is the vector of unknowns of the final algebraic system solved
%    by Manlab. 
% 
%    Idisp contains the indices of the entries of the initial physical 
%    unknown vector u(t) to be displayed. Since the manifold is
%    displayed in 3D, length(Idisp) must be 3. 
%    If Idisp=[1 3 2], u1 is the x-axis, u2 is the y-axis and u2 is
%    the zaxis. 
%
%    Ia contains the indices of the arclength of the branch of
%    periodic solutions to be displayed. It correspond to the
%    radial coordinate of the displayed manifold. 
%    Ia must be such that min(Ia)>=1 and max(Ia)<=size(U,2)
%    
%    H is the number of harmonics of the simulation.
%
%    Neq is the number of equations of the initial system of unknown
%    u(t)
% 
%    fig is a figure/axis handle to plot the manifold on.
%
%    It returns in Hand the handle to the displayed surf object. 
%
%    Hand = plotNNMHBM(Neq,H,U,Idisp,Ia,fig,Nech) does the same
%    with Nech points to discretize one period of the periodic
%    orbits. 
%
%    Hand = plotNNMHBM(Neq,H,U,Idisp,Ia,fig,tt) does the same with
%    tt being used as a time vector to compute the periodic orbits.
%    tt=linspace(0,1,100) means 100 points on one period.
%    tt=linspace(0,.75,100) means 100 points in 3 quarters of one period.
%    tt should be such that min(tt)=0 max(tt)<=1. 
%
%    Example: Hand=plotNNMHBM(Neq,H,U,[1 3 2],1:10:200,1)
%             Hand=plotNNMHBM(Neq,H,U,[2 4 1],1:10:200,1)
%             Hand=plotNNMHBM(Neq,H,U,[2 4 1],1:10:200,1,180)
%             Hand=plotNNMHBM(Neq,H,U,[2 4 1],1:10:200,1,linspace(0,0.75,100))
%
%    By O. Thomas / Oct. 2010


function Hand=plotNNMHBM(Neq,H,U,Idisp,Ia,fig,varargin)


if length(Idisp)~=3
  disp('length(Idisp) should be 3')
  Hand=[];
  return
end 


% Activate axis
% =============

if floor(fig)==fig
  figure(fig)
%  clf
  hold on
else
  axes(fig)
  hold on
end

% Quelques tests
% ==============

if isempty(varargin)
  Nech=360;
  tt=linspace(0,1,Nech);
  tt=tt';
elseif length(varargin{1})==1
  Nech=varargin{1};
  tt=linspace(0,1,Nech);
  tt=tt';
else
  tt=varargin{1};
end


Ndisp=length(Idisp);
Na=length(Ia);
Nech=length(tt);

if size(tt,1)<size(tt,2)
  tt=tt';
end

if min(Ia)>size(U,2)
  disp('min(Ia) should be <= size(U,2)')
  return
end
 
if max(Ia)>size(U,2)
  disp('max(Ia) should be <= size(U,2)')
  return
end

% Calcul des orbites priodiques
% ==============================

Uplot=zeros(Nech,Na,Ndisp);

for ii=1:Ndisp
  idisp=Idisp(ii);
  for jj=1:Na
    ia=Ia(jj);
    Iloc=idisp:Neq:(2*H+1)*Neq;
    Uloc=U(Iloc,ia);
    utt=calc_periodic(Uloc,tt);
    Uplot(:,jj,ii)=utt;
  end
end


% Coloriage de la surface
% =======================

Ucol=Uplot(:,:,3); % En fonction de la position sur l'axe des z
Ucol=ones(Nech,1)*(1:Na); % En fonction de l'indice de l'orbite priodique

% En fonction de l'abscisse curviligne ss sur la ligne (u1,u2,u3)
% dfinie ci-aprs.
ind=floor(Nech/2);
u1=Uplot(ind,:,1);
u2=Uplot(ind,:,2);
u3=Uplot(ind,:,3);
ss=sqrt((u1(2:end)-u1(1:end-1)).^2+(u2(2:end)-u2(1:end-1)).^2+(u3(2:end)-u3(1:end-1)).^2);
ss=cumsum(ss);
ss=[0 ss];
Ucol=ones(Nech,1)*ss;

%size(Ucol)
%size(Uplot(:,:,1))

% Affichage
% =========


%subplot(2,1,1)
%hold on
%for ii=1:Na
%  plot3(Uplot(:,ii,1),Uplot(:,ii,2),Uplot(:,ii,3))
%end
%xlabel(['u_',num2str(Idisp(1))])
%ylabel(['u_',num2str(Idisp(2))])
%zlabel(['u_',num2str(Idisp(3))])
%view(-20,40)

%subplot(2,1,2)
%Hand=surf(Uplot(:,:,1),Uplot(:,:,2),Uplot(:,:,3),Uplot(:,:,3));%,'FaceColor',[0.9  0.5 0],'EdgeColor','k','FaceLighting','phong')
Hand=surf(Uplot(:,:,1),Uplot(:,:,2),Uplot(:,:,3),Ucol);
set(Hand,'Edgecolor','none')

axis tight
view(-50,30)
lighting gouraud
Hchild=get(gca,'Children');

for ii=1:length(Hchild)
  hc=Hchild(ii);
  if strcmp(get(hc,'type'),'light')
    delete(hc);
  end
end
camlight('left');

grid on

xlabel(['u_',num2str(Idisp(1))])
ylabel(['u_',num2str(Idisp(2))])
zlabel(['u_',num2str(Idisp(3))])
view(-40,40)
box on



function utt=calc_periodic(U,tt);
% Cette fonction calcule une serie temporelle 
% periodique, connaissant l'amplitude de ses
% harmonique en sinus et cosinus.
%
% u(t)=U0 + U1c cos t + U1s sin t + ... + UHc cos H*t + UHs sin H*t
% U=[U0 U1c U1s U2c U2s ... UHc UHs]

L=length(U);
H=(L-1)/2;
utt=ones(size(tt))*U(1);

for hh=1:H
  utt=utt+U(2*hh)*cos(2*pi*hh*tt)+U(2*hh+1)*sin(2*pi*hh*tt);
end


