% plotbranchbifXHBM   Branch plot for Manlab/HBM with user defined x axis.
%    Hand=plotbranchbifXHBM(H,Neq,Nds,U,Ustab,X,Idisp,Hdisp,fig) plots the 
%    amplitude of the harmonics components of the solution computed by the
%    Manlab/HBM method as a function of vector X.
%    It also displays the bifurcation points:
%    - B for a simple bifurcation (saddle-node, pitchfork...)
%    - PD for period doubling bifurcation
%    - NS for Neimark-Sacker bifurcation
%
%    U is the vector of unknowns of the final algebraic system solved
%    by Manlab and Ustab the stability data. 
% 
%    X is a vector, with  length(X)=size(U,2)
%
%    Idisp contains the indices of the entries of the initial physical 
%    unknown vector u(t) to be displayed and Hdisp the corresponding
%    harmonics number.
%
%    H is the number of harmonics of the simulation.
%
%    Neq is the number of equations of the initial system of unknown
%    u(t)
%
%    Nds is the initial dynamical system size (this is also the number
%    of Floquet multipliers)
%
%    fig is a figure/axis handle to plot the bargraph on.
%
%    It returns in Hand a column vector of handles to lineseries objects, 
%    one handle per plotted line.
%
%    Example: Hand=plotbranchbifHBM(obj.H,obj.Neq,2,U,Ustab,PERIOD,[1 1 1],[1 3 5],1)
%             plots the three first odd harmonics (H=1, 3, 5) of u1. 
%             as a function of the PERIOD vector
%
%    By O. Thomas / Oct. 2010

function Hand=plotbranchbifXHBM(H,Neq,Nds,U,Ustab,X,Idisp,Hdisp,fig)

pencolorML;
style = {':','-'};

% Activate axis
% =============

if floor(fig)==fig
  figure(fig)
  hold on
else
  axes(fig)
  hold on
end

% Plotting the branches
% =====================

%  0 1 1 1 1 1  1 0 0 0 0 0 0
%0 1 1 1 1 1 1  0 0 0 0 0 0
%  1 0 0 0 0 0 -1 0 0 0 0 0

Nb=length(Idisp);
Hand=[];
Handleg=[];
hold on

%omega = U(end,:);
%stab = Ustab(1,:);  
[Bbif,stab]=detectbifurcHBM(Nds,U,Ustab);
Ibif=[1 Bbif(1,:) size(U,2)];
Tbif=Bbif(2,:);

for ib=1:Nb
  idisp=Idisp(ib);
  hdisp=Hdisp(ib);
  colloc=Col(ib,:);
  if hdisp==0
    I0=idisp;
    Amp=U(I0,:);
  else
    Icos=Neq+idisp+(hdisp-1)*2*Neq;
    Amp=sqrt(U(Icos,:).^2+U(Icos+Neq,:).^2);
  end
  for bb=1:(length(Ibif)-1)
    II=Ibif(bb):Ibif(bb+1);
    st=style{stab(Ibif(bb)+1)+1};
    hand=plot(X(II),Amp(II),st,'Color',colloc,'linewidth',1);
    Hand=[Hand;hand];
    if stab(Ibif(bb)+1)==1
      Handleg(ib)=hand;
    end
  end
  if length(Handleg)<ib
    Handleg(ib)=hand;
  end
  plot(X(Ibif(2:end-1)),Amp(Ibif(2:end-1)),'.','Color',colloc)
  Leg{ib}=['H_',num2str(hdisp),'u_',num2str(idisp)];
  for ib=1:size(Bbif,2)
    ibif=Bbif(1,ib);
    tbif=Bbif(2,ib);
    switch tbif
     case 1
      str='B';
     case 2
      str='PD';
     case 3
      str='NS';
    end
    text(X(ibif),Amp(ibif),str,'margin',3,'verticalalignment','Bottom')
  end
end
legend(Handleg,Leg,'location','best')
box on
ylabel('Harmonics amplitude')
xlabel('Bifurcation parameter')
  
  
